@@ -18,7 +18,7 @@ use Image::CairoSVG;
 use Module::Path 'module_path';
 use PDF::Cairo::Util;
 
-our $VERSION = "1.04";
+our $VERSION = "1.05";
 $VERSION = eval $VERSION;
 
 =head1 NAME
@@ -806,17 +806,62 @@ array reference.
 
 sub rel_rect {
 	my $self = shift;
-	my ($dx, $dy, $w, $h) = @_;
-	($dx, $dy, $w, $h) = @$dx if ref $dx eq 'ARRAY';
-	if ($self->{context}->has_current_point) {
-		my $tmp = $self->{context}->get_matrix;
-		my ($x, $y) = $self->{context}->get_current_point;
-		$self->translate($x, - $y);
-		$self->rect($dx, $dy, $w, $h);
-		$self->{context}->set_matrix($tmp);
-	}else{
-		croak "PDF::Cairo::rel_rect: no current point";
+	croak "PDF::Cairo::rel_rect: no current point"
+		unless $self->{context}->has_current_point;
+	my $tmp = $self->{context}->get_matrix;
+	my ($x, $y) = $self->{context}->get_current_point;
+	$self->translate($x, - $y);
+	$self->rect(@_);
+	$self->{context}->set_matrix($tmp);
+	return $self;
+}
+
+=item B<roundrect> $x, $y, $width, $height, [$radius]
+
+Draws a rectangle starting at ($x, $y) with $width and $height,
+with corners rounded by $radius (defaults to 1/20th the length
+of the shortest side).
+
+=cut
+
+sub roundrect {
+	my $self = shift;
+	my ($x, $y, $w, $h, $r) = @_;
+	my $tmp = $w < $h ? $w : $h;
+	$r ||= $tmp / 20;
+	if ($r * 2 > $tmp) {
+		$r = $tmp / 2;
+		carp "PDF::Cairo::roundrect: Radius too large, reduced to $r";
 	}
+	$self->arc($x + $r, $y + $r,
+		$r, $r, -90, -180, 1);
+ 	$self->arc($x + $r, $y + $h - $r,
+		$r, $r, 180, 90);
+	$self->arc($x + $w - $r, $y + $h - $r,
+		$r, $r, 90, 0);
+	$self->arc($x + $w - $r, $y + $r,
+		$r, $r, 0, -90);
+	$self->close;
+	return $self;
+}
+
+=item B<rel_roundrect> $dx, $dy, $width, $height, [$radius]
+
+Draws a rectangle offset from the current point by ($dx, $dy), with
+$width and $height, with corners rounded by $radius (defaults to
+1/20th the length of the shortest side).
+
+=cut
+
+sub rel_roundrect {
+	my $self = shift;
+	croak "PDF::Cairo::rel_roundrect: no current point"
+		unless $self->{context}->has_current_point;
+	my $tmp = $self->{context}->get_matrix;
+	my ($x, $y) = $self->{context}->get_current_point;
+	$self->translate($x, - $y);
+	$self->roundrect(@_);
+	$self->{context}->set_matrix($tmp);
 	return $self;
 }
 
@@ -975,15 +1020,16 @@ sub loadfont {
 	return PDF::Cairo::Font->new($self, $face);
 }
 
-=item B<setfont> $fontref, $size
+=item B<setfont> $fontref, [$size]
 
-Set the current font and size.
+Set the current font and size. Default size is 1 point.
 
 =cut
 
 sub setfont {
 	my $self = shift;
 	my ($font, $size) = @_;
+	$size ||= 1;
 	$self->{context}->set_font_face($font->{face})
 		if ref $font and $font->{face} ne $self->{context}->get_font_face;
 	$self->{context}->set_font_size($size);
@@ -1013,6 +1059,8 @@ sub setfontsize {
 
 =item shift => $vertical_shift
 
+=item center => 1
+
 =back
 
 Display text at current position, with current font and fillcolor. If
@@ -1031,20 +1079,24 @@ sub print {
 		$self->_api2_print(@_);
 	}else{
 		my ($text, %options) = @_;
-		my $extents = $self->{context}->text_extents($text);
-		my $width = $extents->{width};
-		my $height = $extents->{height};
+		my $extents = $self->extents($text);
+		my $width = $extents->width;
+		my $height = $extents->height;
 		my ($dx, $dy) = (0, 0);
 		$options{align} ||= "left";
+		if ($options{center}) {
+			$options{align} = 'center';
+			$options{valign} = 'center';
+		}
 		if ($options{align} eq "center") {
-			$dx -= $width / 2;
+			$dx -= $width / 2 + $extents->x;
 		}elsif ($options{align} eq "right") {
 			$dx -= $width;
 		}
 		# TODO: add "bottom", distinct from default baseline
 		$options{valign} ||= "baseline";
 		if ($options{valign} eq "center") {
-			$dy += $height / 2;
+			$dy += $height / 2 + $extents->y;
 		}elsif ($options{valign} eq "top") {
 			$dy += $height;
 		}
@@ -1066,6 +1118,47 @@ sub print {
 	return $self;
 }
 
+=item B<autosize> $text, $box
+
+Set the size of the current font to the largest value that allows
+$text to fit inside of the specified PDF::Cairo::Box object.
+
+=cut
+
+sub autosize {
+	my $self = shift;
+	my ($text, $box) = @_;
+	croak "PDF::Cairo::autosize: requires a PDF::Cairo::Box object"
+		unless ref $box eq 'PDF::Cairo::Box';
+	$self->setfontsize($box->height);
+	my $extents = $self->extents($text);
+	$self->setfontsize($box->height * $box->width / $extents->width)
+		if $extents->width > $box->width;
+	return $self;
+}
+
+=item B<extents> $text
+
+Returns a L<PDF::Cairo::Box> object containing the ink extents
+of $text as it would be rendered with the current font/size.
+
+=cut
+
+sub extents {
+	my $self = shift;
+	my $text = shift;
+	croak "PDF::Cairo::extents: requires text string as argument"
+		unless defined $text;
+	my $extents = $self->{context}->text_extents($text);
+	my $box = PDF::Cairo::Box->new(
+		width => $extents->{width},
+		height => $extents->{height},
+		x => $extents->{x_bearing},
+		y => -($extents->{y_bearing} + $extents->{height}),
+	);
+	return $box;
+}
+
 =item B<textpath> $text
 
 Add the outlines of the glyphs in $text to the current path, using
@@ -1460,7 +1553,8 @@ sub place {
 Creates a new PDF::Cairo recording object. You can draw on it
 normally, but can only access the results with place(). Options are
 the same as new(). height() and width() methods are available to
-determine appropriate scaling values.
+determine appropriate scaling values. Recording surfaces are clipped
+to their size.
 
 =cut
 
@@ -1761,9 +1855,11 @@ sub _api2_print {
 	my $text = join(' ', @text);
 	$self->{context}->set_font_face($font->{face});
 	$self->{context}->set_font_size($size);
-	my $width = $self->{context}->text_extents($text)->{width};
+	my $extents = $self->{context}->text_extents($text);
+	my $width = $extents->{width};
+	my $x_bearing = $extents->{x_bearing};
 	if ($justification == 1) {
-		$x -= $width / 2;
+		$x -= $width / 2 + $x_bearing;
 	}elsif ($justification == 2) {
 		$x -= $width;
 	}
